/*
 * Part of the ROBOID project - http://hamster.school
 * Copyright (C) 2016 Kwang-Hyun Park (akaii@kw.ac.kr)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
*/

#ifndef _LIBRARY_H_
#define _LIBRARY_H_

typedef void (*_SCAN)(void);
typedef void (*_SET_EXECUTABLE)(void (*execute)(void* arg), void* arg);
typedef void (*_WAIT)(int milliseconds);
typedef void (*_WAIT_UNTIL)(int (*evaluate)(void* arg), void* arg);
typedef void (*_WAIT_UNTIL_READY)(void);
typedef void (*_DISPOSE_ALL)(void);

typedef struct hamster* (*_HAMSTER_CREATE)(void);
typedef struct hamster* (*_HAMSTER_CREATE_PORT)(const char* port_name);
typedef const char* (*_HAMSTER_GET_NAME)(void);
typedef void (*_HAMSTER_SET_NAME)(const char* name);
typedef const char* (*_HAMSTER_GET_ID)(void);
typedef int (*_HAMSTER_E)(int device_id);
typedef int (*_HAMSTER_READ)(int device_id);
typedef int (*_HAMSTER_READ_AT)(int device_id, int index);
typedef int (*_HAMSTER_READ_ARRAY)(int device_id, int* data, int length);
typedef float (*_HAMSTER_READ_FLOAT)(int device_id);
typedef float (*_HAMSTER_READ_FLOAT_AT)(int device_id, int index);
typedef int (*_HAMSTER_READ_FLOAT_ARRAY)(int device_id, float* data, int length);
typedef int (*_HAMSTER_WRITE)(int device_id, int data);
typedef int (*_HAMSTER_WRITE_AT)(int device_id, int index, int data);
typedef int (*_HAMSTER_WRITE_ARRAY)(int device_id, const int* data, int length);
typedef int (*_HAMSTER_WRITE_FLOAT)(int device_id, float data);
typedef int (*_HAMSTER_WRITE_FLOAT_AT)(int device_id, int index, float data);
typedef int (*_HAMSTER_WRITE_FLOAT_ARRAY)(int device_id, const float* data, int length);
typedef void (*_HAMSTER_RESET)(void);
typedef void (*_HAMSTER_DISPOSE)(void);
typedef void (*_HAMSTER_WHEELS)(double left_speed, double right_speed);
typedef void (*_HAMSTER_LEFT_WHEEL)(double speed);
typedef void (*_HAMSTER_RIGHT_WHEEL)(double speed);
typedef void (*_HAMSTER_STOP)(void);
typedef void (*_HAMSTER_LINE_TRACER_MODE)(int mode);
typedef void (*_HAMSTER_LINE_TRACER_SPEED)(double speed);
typedef void (*_HAMSTER_BOARD_FORWARD)(void);
typedef void (*_HAMSTER_BOARD_LEFT)(void);
typedef void (*_HAMSTER_BOARD_RIGHT)(void);
typedef void (*_HAMSTER_LEDS)(int left_color, int right_color);
typedef void (*_HAMSTER_LEFT_LED)(int color);
typedef void (*_HAMSTER_RIGHT_LED)(int color);
typedef void (*_HAMSTER_BEEP)(void);
typedef void (*_HAMSTER_BUZZER)(double hz);
typedef void (*_HAMSTER_TEMPO)(double bpm);
typedef void (*_HAMSTER_PITCH)(double pitch);
typedef void (*_HAMSTER_NOTE)(double pitch, double beats);
typedef void (*_HAMSTER_IO_MODE_A)(int mode);
typedef void (*_HAMSTER_IO_MODE_B)(int mode);
typedef void (*_HAMSTER_OUTPUT_A)(double value);
typedef void (*_HAMSTER_OUTPUT_B)(double value);
typedef int (*_HAMSTER_SIGNAL_STRENGTH)(void);
typedef int (*_HAMSTER_LEFT_PROXIMITY)(void);
typedef int (*_HAMSTER_RIGHT_PROXIMITY)(void);
typedef int (*_HAMSTER_LEFT_FLOOR)(void);
typedef int (*_HAMSTER_RIGHT_FLOOR)(void);
typedef int (*_HAMSTER_ACCELERATION_X)(void);
typedef int (*_HAMSTER_ACCELERATION_Y)(void);
typedef int (*_HAMSTER_ACCELERATION_Z)(void);
typedef int (*_HAMSTER_LIGHT)(void);
typedef int (*_HAMSTER_TEMPERATURE)(void);
typedef int (*_HAMSTER_INPUT_A)(void);
typedef int (*_HAMSTER_INPUT_B)(void);

_SCAN scan;
_SET_EXECUTABLE set_executable;
_WAIT wait;
_WAIT_UNTIL wait_until;
_WAIT_UNTIL_READY wait_until_ready;
_DISPOSE_ALL dispose_all;

_HAMSTER_CREATE hamster_create;
_HAMSTER_CREATE_PORT hamster_create_port;
_HAMSTER_GET_NAME hamster_get_name;
_HAMSTER_SET_NAME hamster_set_name;
_HAMSTER_GET_ID hamster_get_id;
_HAMSTER_E hamster_e;
_HAMSTER_READ hamster_read;
_HAMSTER_READ_AT hamster_read_at;
_HAMSTER_READ_ARRAY hamster_read_array;
_HAMSTER_READ_FLOAT hamster_read_float;
_HAMSTER_READ_FLOAT_AT hamster_read_float_at;
_HAMSTER_READ_FLOAT_ARRAY hamster_read_float_array;
_HAMSTER_WRITE hamster_write;
_HAMSTER_WRITE_AT hamster_write_at;
_HAMSTER_WRITE_ARRAY hamster_write_array;
_HAMSTER_WRITE_FLOAT hamster_write_float;
_HAMSTER_WRITE_FLOAT_AT hamster_write_float_at;
_HAMSTER_WRITE_FLOAT_ARRAY hamster_write_float_array;
_HAMSTER_RESET hamster_reset;
_HAMSTER_DISPOSE hamster_dispose;
_HAMSTER_WHEELS hamster_wheels;
_HAMSTER_LEFT_WHEEL hamster_left_wheel;
_HAMSTER_RIGHT_WHEEL hamster_right_wheel;
_HAMSTER_STOP hamster_stop;
_HAMSTER_LINE_TRACER_MODE hamster_line_tracer_mode;
_HAMSTER_LINE_TRACER_SPEED hamster_line_tracer_speed;
_HAMSTER_BOARD_FORWARD hamster_board_forward;
_HAMSTER_BOARD_LEFT hamster_board_left;
_HAMSTER_BOARD_RIGHT hamster_board_right;
_HAMSTER_LEDS hamster_leds;
_HAMSTER_LEFT_LED hamster_left_led;
_HAMSTER_RIGHT_LED hamster_right_led;
_HAMSTER_BEEP hamster_beep;
_HAMSTER_BUZZER hamster_buzzer;
_HAMSTER_TEMPO hamster_tempo;
_HAMSTER_PITCH hamster_pitch;
_HAMSTER_NOTE hamster_note;
_HAMSTER_IO_MODE_A hamster_io_mode_a;
_HAMSTER_IO_MODE_B hamster_io_mode_b;
_HAMSTER_OUTPUT_A hamster_output_a;
_HAMSTER_OUTPUT_B hamster_output_b;
_HAMSTER_SIGNAL_STRENGTH hamster_signal_strength;
_HAMSTER_LEFT_PROXIMITY hamster_left_proximity;
_HAMSTER_RIGHT_PROXIMITY hamster_right_proximity;
_HAMSTER_LEFT_FLOOR hamster_left_floor;
_HAMSTER_RIGHT_FLOOR hamster_right_floor;
_HAMSTER_ACCELERATION_X hamster_acceleration_x;
_HAMSTER_ACCELERATION_Y hamster_acceleration_y;
_HAMSTER_ACCELERATION_Z hamster_acceleration_z;
_HAMSTER_LIGHT hamster_light;
_HAMSTER_TEMPERATURE hamster_temperature;
_HAMSTER_INPUT_A hamster_input_a;
_HAMSTER_INPUT_B hamster_input_b;

void roboid_load_library(void);
void roboid_free_library(void);

#endif
